use ezcad::{array_of::ArrayOf, layer::Layer, pen::Pen};
use serde::{Deserialize, Serialize};

use self::{
    object::{DeleteObjects, ExportObject, ImportObject, RectangleArray},
    pen::{ClonePen, ImportExportPen, PatchPen, PatternPen},
};

pub mod object;
pub mod pen;

#[derive(Debug, Serialize, Deserialize)]
pub enum Operation {
    PatchPen(PatchPen),
    ClonePen(ClonePen),
    ClonePenRange(ClonePen),
    PatternPen(PatternPen),
    ExportPen(ImportExportPen),
    ImportPen(ImportExportPen),
    ExportObject(ExportObject),
    ImportObject(ImportObject),
    DeleteObjects(DeleteObjects),
    RectangleArray(RectangleArray),
}

pub trait Operations {
    fn apply(&self, pens: &mut Vec<Pen>, layers: &mut ArrayOf<Layer>);
}

impl Operations for Vec<Operation> {
    fn apply(&self, pens: &mut Vec<Pen>, layers: &mut ArrayOf<Layer>) {
        for op in self {
            match op {
                Operation::PatchPen(x) => x.patch(pens),
                Operation::ClonePen(x) => x.clone_single(pens),
                Operation::ClonePenRange(x) => x.clone_range(pens),
                Operation::PatternPen(x) => x.pattern(pens),
                Operation::ImportPen(x) => x.import(pens),
                Operation::ExportPen(x) => x.export(pens),
                Operation::ExportObject(x) => x.export(layers),
                Operation::ImportObject(x) => x.import(layers),
                Operation::DeleteObjects(x) => x.delete(layers),
                Operation::RectangleArray(x) => x.generate(pens, layers),
            }
        }
    }
}

#[derive(Debug, Serialize, Deserialize)]
#[serde(rename_all = "PascalCase")]
pub struct Config {
    pub ops: Vec<Operation>,
}
